!define VERSION "4.1.0"
!define REGROOT "Software\Itefix\ICW"
!define NAME "cwRsyncServer"
!define UNINSTPROG "uninstall_${NAME}.exe"
!define DEF_SERVICE_ACCOUNT "SvcCWRSYNC"

!define BASE_PACKAGE "ICW_Base_2.1.6_installer.exe"
!define BASE_UNINSTALL "uninstall_ICW_Base.exe"
!define RSYNCSERVER_NAME "RsyncServer"
!define SVCNAME "RsyncServer"
!define RSYNCSERVER_PACKAGE "ICW_RsyncServer_2.0.3_installer.exe"
!define RSYNCSERVER_UNINSTALL "uninstall_ICW_RsyncServer.exe"

SetCompressor /SOLID LZMA
InstallDir $PROGRAMFILES\ICW
InstallDirRegKey HKLM ${REGROOT} "InstallDirectory"
Name "${NAME} ${VERSION}"
OutFile "${NAME}_${VERSION}_Installer.exe"

VIAddVersionKey  "ProductName" "${NAME}"
VIAddVersionKey  "CompanyName" "ITeF!x Consulting"
VIAddVersionKey  "FileDescription" "${NAME}"
VIAddVersionKey  "FileVersion" "${VERSION}"
VIProductVersion "${VERSION}.0"

!include MUI.nsh
!define MUI_ICON "cwrsync.ico"
!define MUI_UNICON "cwrsync-uninstall.ico"

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "LICENSE.TXT"
!insertmacro MUI_PAGE_DIRECTORY
Page custom ServiceAccountGUI ServiceAccountGUIPageLeave
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_LANGUAGE "English"

!include nsDialogs.nsh
!include LogicLib.nsh
!include WinVer.nsh
!include WinMessages.nsh
!include FileFunc.nsh

!insertmacro GetParameters
!insertmacro GetOptions

Var svcuser
Var svcpassword

Var hDialog
Var hsvcaccount
Var hsvcpassword1
Var hsvcpassword2

Function .onInit

	StrCpy $svcuser ""
	StrCpy $svcpassword ""

	${IfNot} ${AtLeastWinNT4}}
	MessageBox MB_OK "NT4 and above required"
	Quit
	${EndIf}

	InitPluginsDir
	File /oname=$PLUGINSDIR\${BASE_PACKAGE} ${BASE_PACKAGE}
	File /oname=$PLUGINSDIR\${RSYNCSERVER_PACKAGE} ${RSYNCSERVER_PACKAGE}
	File /oname=$PLUGINSDIR\pwdgen.exe pwdgen.exe
	
	# Check if user / password is defined via command line
	${GetParameters} $0
	${GetOptions} $0 "/u="  $svcuser
	${GetOptions} $0 "/p="  $svcpassword
	
	# Check if user/password is already supplied
	StrCmp $svcuser "" Init_Cont_A 0
	StrCmp $svcpassword "" Init_Cont_A 0
	Goto Init_Cont_B
	
Init_Cont_A:
	# Create default values
	nsExec::ExecToStack "$PLUGINSDIR\pwdgen.exe"
	Pop $0
	Pop $0
	
	StrCpy $svcpassword $0 14
	StrCpy $svcuser ${DEF_SERVICE_ACCOUNT}

Init_Cont_B:

FunctionEnd

# Install section
Section "cwRsync"

	StrCpy $0 ""
	IfSilent 0 +2
	StrCpy $0 "/S"
	
	WriteRegStr HKLM ${REGROOT} "InstallDirectory" "$INSTDIR"
	
	DetailPrint "Stop ${SVCNAME} service"
	nsExec::Exec '"$INSTDIR\Bin\cygrunsrv" -E ${SVCNAME}'
	
	ExecWait '"$PLUGINSDIR\${BASE_PACKAGE}" $0'
	ExecWait '"$PLUGINSDIR\${RSYNCSERVER_PACKAGE}" $0 /u=$svcuser /p=$svcpassword'
	
	; Set installation Directory
	ReadRegStr $INSTDIR HKLM ${REGROOT} "InstallDirectory"
	
	SetOutPath $INSTDIR
	WriteUninstaller ${UNINSTPROG}
		
	DetailPrint "Creating shortcuts"
	CreateDirectory "$SMPROGRAMS\${NAME}"
	SetOutPath $INSTDIR
	CreateShortCut "$SMPROGRAMS\${NAME}\01. rsyncd.conf.lnk" "notepad.exe" "$INSTDIR\rsyncd.conf"
	CreateShortCut "$SMPROGRAMS\${NAME}\02. Prep a Dir for Upload.lnk" "$INSTDIR\Bin\PrepUploadDir.exe"
	CreateShortCut "$SMPROGRAMS\${NAME}\03. Web site.lnk" "http://itefix.no/cwrsync"
	CreateShortCut "$SMPROGRAMS\${NAME}\10. DONATE!.lnk" "http://itefix.no/SupportItefix.html"
	CreateDirectory "$SMPROGRAMS\${NAME}\Documentation"
	CreateShortCut "$SMPROGRAMS\${NAME}\Documentation\rsync.lnk" "$INSTDIR\doc\rsync.html"
	CreateShortCut "$SMPROGRAMS\${NAME}\Documentation\rsyncd.conf.lnk" "$INSTDIR\doc\rsyncd.conf.html"

	CreateShortCut "$SMPROGRAMS\${NAME}\11. Uninstall cwRsync Server.lnk" "$INSTDIR\${UNINSTPROG}"

	; Write the uninstall keys for Windows
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "DisplayName" "${NAME} (remove only)"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "UninstallString" '"$INSTDIR\${UNINSTPROG}"'

SectionEnd

# Uninstall section
Section "Uninstall"

	# Uninstall if only base + rsync packages exist, warn otherwise

	DetailPrint "Remove uninstaller"
	Delete $INSTDIR\${UNINSTPROG}
	
	StrCpy $0 ""
	IfSilent 0 +2
	StrCpy $0 "/S"
	
	ExecWait '"$INSTDIR\${RSYNCSERVER_UNINSTALL}" $0'
	ExecWait '"$INSTDIR\${BASE_UNINSTALL}" $0'

	
	DetailPrint "Remove uninstall registry keys"
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}"
	
	DetailPrint "Remove shortcuts"
	RmDir /r "$SMPROGRAMS\${NAME}"

SectionEnd

Function ServiceAccountGUI

	# Skip dialog if the service account is already defined for OpenSSH package (upgrade)
	ReadRegStr $0 HKLM "${REGROOT}\${RSYNCSERVER_NAME}" "ServiceAccount"	
	StrCmp $0 "" 0 GUI_Serv_Acc_End

	!insertmacro MUI_HEADER_TEXT "Service Account" "Rsync server will be setup as a windows service with the logon credentials below:"

	nsDialogs::Create /NOUNLOAD 1018
	Pop $hDialog

	${If} $hDialog == error
		Abort
	${EndIf}

	${NSD_CreateLabel} 10 2 95% 80 "${NAME} requires a dedicated service account for full functionality. You can either accept the values suggested by the installer (user: $svcuser, password: $svcpassword) or specify your own values. Existing accounts are allowed. In all cases, the account will be entitled with the user privilege 'logon as service'."
	
	${NSD_CreateLabel} 10 90 100 24 "Service account:"
	${NSD_CreateText} 110 90 160 24 ""
	Pop $hsvcaccount
	${NSD_SetText} $hsvcaccount $svcuser
	
	${NSD_CreateLabel} 10 125 100 24 "Type password:"
	${NSD_CreatePassword} 110 125 160 24 ""
	Pop $hsvcpassword1
	${NSD_SetText} $hsvcpassword1 $svcpassword
	
	${NSD_CreateLabel} 10 160 100 24 "Confirm password:"
	${NSD_CreatePassword} 110 160 160 24 ""
	Pop $hsvcpassword2
	${NSD_SetText} $hsvcpassword2 $svcpassword
	
	${NSD_SetFocus} $hsvcaccount

	nsDialogs::Show
	
	GUI_Serv_Acc_End:

FunctionEnd

Function ServiceAccountGUIPageLeave

	# Skip dialog if the service account is already defined for OpenSSH package
	ReadRegStr $0 HKLM "${REGROOT}\${RSYNCSERVER_NAME}" "ServiceAccount"	
	StrCmp $0 "" 0 Leave_Serv_Acc_End

	${NSD_GetText} $hsvcaccount $svcuser
	${NSD_GetText} $hsvcpassword1 $1
	${NSD_GetText} $hsvcpassword2 $2

	StrCmp $1 $2 +3
	MessageBox MB_OK|MB_ICONSTOP "Passwords do not match. Please start the installer again."
	Quit
	
	StrCpy $svcpassword $1

	Leave_Serv_Acc_End:
FunctionEnd
